<?php

namespace Drupal\openstory\Controller;

use Drupal\Core\Access\AccessibleInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\jsonapi\Controller\RequestHandler;
use Drupal\jsonapi\Exception\EntityAccessDeniedHttpException;
use Drupal\jsonapi\Normalizer\EntityNormalizer;
use Drupal\jsonapi\Normalizer\JsonApiDocumentTopLevelNormalizer;
use Drupal\jsonapi\Resource\EntityCollection;
use Drupal\jsonapi\Resource\JsonApiDocumentTopLevel;
use Drupal\jsonapi\ResourceResponse;
use Drupal\jsonapi\ResourceType\ResourceType;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Serializer\Serializer;
use Drupal\jsonapi\LinkManager\LinkManager;
use Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;

/**
 * Class OsGetAllNodes.
 *
 * @package Drupal\openstory\Controller
 */
class OsGetAllNodes extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user query param.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The JSON API resource type repository.
   *
   * @var \Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface
   */
  protected $resourceTypeRepository;

  /**
   * The link manager to get the links.
   *
   * @var \Drupal\jsonapi\LinkManager\LinkManager
   */
  protected $linkManager;

  /**
   * The serializer.
   *
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * OsGetAllNodes constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Symfony\Component\HttpFoundation\Request $currentRequest
   *   The current request.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, Request $currentRequest, LinkManager $link_manager, ResourceTypeRepositoryInterface $resourceTypeRepository, Serializer $serializer) {
    $this->entityTypeManager = $entityTypeManager;
    $this->currentRequest = $currentRequest;
    $this->linkManager = $link_manager;
    $this->resourceTypeRepository = $resourceTypeRepository;
    $this->serializer = $serializer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('jsonapi.link_manager'),
      $container->get('jsonapi.resource_type.repository'),
      $container->get('serializer')
    );
  }

  /**
   * Returns information about all entity types on the systems.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   Return the definition via jsonapi.
   */
  public function getAllNodes() {
    //$osEntitiesList = new OsEntitiesList($this->entityTypeManager, $this->currentRequest);
   // $entities = $osEntitiesList->entitiesList('node');


//    $result = [];
//    $entity_storage = $this->entityTypeManager->getStorage('node');
//    $query = $entity_storage->getQuery();
//    $query->sort('created', 'DESC');
//    $nids = $query->execute();
//    foreach ($entity_storage->loadMultiple($nids) as $node) {
//      $normalizedNode = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($node);
//      if (!empty($normalizedNode)) {
//        $result[] = $normalizedNode;
//      }
//    }
//    return new ResourceResponse($result, 200);


    //    $node = $this->entityTypeManager->getStorage('node')
    //      ->create(['type' => 'bodsy']);
    //    $normalizedNode = \Drupal::service('jsonapi.entity.to_jsonapi')
    //      ->normalize($node);
    //    $d = NULL;

        $entity_storage = $this->entityTypeManager->getStorage('node');

        $query = $entity_storage->getQuery();
        $r = $query->execute();

        $collection_data = [];
        $entities = [];
        foreach ($entity_storage->loadMultiple($r) as $entity) {
          $entities[] = $entity->getOriginalObject()->getValue();
          $collection_data[$entity->id()] = static::getEntityAndAccess($entity);
        }
    //    //$collection_data = $this->loadEntitiesWithAccess($entity_storage, $r);
    //    $entity_collection = new EntityCollection(array_column($collection_data, 'entity'));
    //    $context = [];
    //    $context['request'] = $this->currentRequest;
    //    $rt = new ResourceType('node', 'bodsy', FieldableEntityInterface::class);
    //    $rt->setRelatableResourceTypes([]);
    //    $context['resource_type'] = $rt;
    //    $a = new JsonApiDocumentTopLevelNormalizer($this->linkManager, $this->entityTypeManager, $this->resourceTypeRepository);
    ////    $s = new \Drupal\jsonapi\Serializer\Serializer();
    ////    $s->setFallbackNormalizer($this->serializer);
    //    $a->setSerializer($this->serializer);
    //    $b = $a->normalize(new JsonApiDocumentTopLevel($entity_collection), 'api_json', $context);
    //    $d =null;
    //    return new ResourceResponse(new JsonApiDocumentTopLevel($entity_collection), 200, []);


    //    $client = \Drupal::httpClient();
    //    try {
    //      $response = $client->get('http://openstory-new.dd:8083/jsonapi/node/bodsy', [
    //        'headers' => ['Content-Type' => 'application/vnd.api+json']
    //      ]);
    //      $data = $response->getBody();
    //    }
    //    catch (RequestException $e) {
    //      watchdog_exception('openstory', $e->getMessage());
    //    }
    ////    if(!empty($entities)) {
    //      $responseData = $entities->getResponseData();
    //      if (!empty($responseData)) {
    //        if (is_array($responseData) && array_key_exists('node', $responseData)) {
    //          $bundles = $responseData['node']['bundles'];
    //          foreach ($bundles as $bundle => $bundleLabel) {
    //
    //            $node = $this->entityTypeManager->getStorage('node')->loadByProperties(['type' => $bundle]);
    //            if (!empty($node)) {
    //              foreach ($node as $item => $a) {
    //                $normalizedNode = \Drupal::service('jsonapi.entity.to_jsonapi')->normalize($node[$item]);
    //                $d =null;
    //              }
    //
    //
    //            }
    //          }
    //        }
    //      }
    //    }
    // $d = null;
  }

  public static function getEntityAndAccess(EntityInterface $entity) {
    /** @var \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository */
    $entity_repository = \Drupal::service('entity.repository');
    $entity = $entity_repository->getTranslationFromContext($entity, NULL, ['operation' => 'entity_upcast']);
    $access = $entity->access('view', NULL, TRUE);
    // Accumulate the cacheability metadata for the access.
    $output = [
      'access' => $access,
      'entity' => $entity,
    ];
    if ($entity instanceof AccessibleInterface && !$access->isAllowed()) {
      // Pass an exception to the list of things to normalize.
      $output['entity'] = new EntityAccessDeniedHttpException($entity, $access, '/data', 'The current user is not allowed to GET the selected resource.');
    }

    return $output;
  }
}
